"""
Contains data classes for the WAS module.
"""

from .WebApp import WebApp
from .Tag import WASTag
from .AuthRecord import AuthRecord
from .CrawlingScript import CrawlingScript
from .PostmanCollection import PostmanCollection
from .SwaggerFile import SwaggerFile
from .UrlEntry import UrlEntry
from .Comment import Comment
from .WebAppAuthRecord import WebAppAuthRecord
from .WebAppAuthFormRecord import WebAppAuthFormRecord
from .WebAppAuthServerRecord import WebAppAuthServerRecord
